@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: client.h,v 1.30 2000/06/01 16:57:46 nisse Exp $ */
+ * $Id: client.h,v 1.31 2000/08/03 23:35:20 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -55,7 +55,7 @@ make_accept_service_handler(UINT32 service,
      (name request_service)
      (super command)
      (vars
-       (service simple int)))
+       (service . int)))
 */
 
 void
