@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: crypto.h,v 1.18 2000/05/01 19:39:39 nisse Exp $ */
+ * $Id: crypto.h,v 1.20 2000/09/10 23:47:50 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -42,6 +42,10 @@ struct crypto_algorithm *make_twofish_algorithm(UINT32 key_size);
 struct crypto_algorithm *make_twofish(void);
 struct crypto_algorithm *make_blowfish_algorithm(UINT32 key_size);
 struct crypto_algorithm *make_blowfish(void);
+struct crypto_algorithm *make_rijndael_algorithm(UINT32 key_size);
+struct crypto_algorithm *make_rijndael(void);
+struct crypto_algorithm *make_serpent_algorithm(UINT32 key_size);
+struct crypto_algorithm *make_serpent(void);
 struct crypto_algorithm *make_des3(void);
 struct crypto_algorithm *make_cast_algorithm(UINT32 key_size);
 struct crypto_algorithm *make_cast(void);
@@ -65,4 +69,4 @@ pkcs5_derive_key(struct mac_algorithm *prf,
 		 UINT32 iterations,
 		 UINT32 key_length, UINT8 *key);
 
-#endif
+#endif /* LSH_CRYPTO_H_INCLUDED */
