@@ -2,7 +2,7 @@
  *
  * Create a packet from a format string and arguments.
  *
- * $Id: format.h,v 1.29 2000/04/23 17:48:49 nisse Exp $ */
+ * $Id: format.h,v 1.31 2000/07/28 08:00:48 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -84,9 +84,25 @@ void ssh_format_write(const char *format, UINT32 length, UINT8 *buffer, ...);
 UINT32 ssh_vformat_length(const char *format, va_list args);
 void ssh_vformat_write(const char *format, UINT32 length, UINT8 *buffer, va_list args);
 
-/* Short cut */
+/* Used for generating strings that can be used with standard libc
+ * functions. It checks that the resulting string doesn't contain any
+ * NUL characters (if it does, NULL is returned), and adds a
+ * termianting NUL character at the end, not included in the string
+ * length. */
+struct lsh_string *ssh_cformat(const char *format, ...);
+
+/* Short cuts */
 #define lsh_string_dup(s) (ssh_format("%lS", (s)))
 
+/* FIXME: The names of these three functions are not quite intuitive.
+ * Perhaps they should be renamed or deleted. */
+
+#if 0
+#define format_cstring(s) (ssh_cformat("%lz", (s)))
+#define make_cstring_l(l, s) (ssh_cformat("%ls", (l), (s)))
+#define make_cstring(s, f) (ssh_cformat((f) ? "%lfS" : "%lS", (s)))
+#endif
+
 struct lsh_string *
 format_cstring(const char *s);
 
